{% load i18n %}

<ul class="nav collapsable-menu {{ site_name|lower }}" id="side-menu">
    {% for item in menu_items %}
    <li class="menu-item {{ item.css_classes }}{% if item.active %} active{% endif %}">
        {% if item.collapsible and item.submenu_items %}
            <button disabled>{{ item.title|capfirst }}</button>
            <ul class="collapsible-content">
                {% for submenu_item in item.submenu_items %}
                    <li class="{{ submenu_item.css_classes }}">
                        <a href="{{ submenu_item.url }}">
                            {{ submenu_item.title|capfirst }}
                        </a>
                    </li>
                {% endfor %}
            </ul>
        {% elif item.url %}
            <a href="{{ item.url }}" >{{ item.title|capfirst }}</a>
        {% endif %}
    </li>
    {% endfor %}
</ul>
